@@ -9,25 +9,33 @@ const Cc = Components.classes;
 const Cu = Components.utils;
 
 Cu.import("resource://gre/modules/Services.jsm");
+Cu.import("resource://gre/modules/TelemetryTimestamps.jsm");
+Cu.import("resource://gre/modules/TelemetryPing.jsm");
+Cu.import("resource://gre/modules/TelemetrySession.jsm");
+Cu.import("resource://gre/modules/TelemetryLog.jsm");
 
 const Telemetry = Services.telemetry;
 const bundle = Services.strings.createBundle(
   "chrome://global/locale/aboutTelemetry.properties");
 const brandBundle = Services.strings.createBundle(
   "chrome://branding/locale/brand.properties");
-const TelemetryPing = Cc["@mozilla.org/base/telemetry-ping;1"].
-  getService(Ci.nsITelemetryPing);
 
-// Maximum height of a histogram bar (in em)
+// Maximum height of a histogram bar (in em for html, in chars for text)
 const MAX_BAR_HEIGHT = 18;
+const MAX_BAR_CHARS = 25;
 const PREF_TELEMETRY_SERVER_OWNER = "toolkit.telemetry.server_owner";
-//@line 27 "/build/bluegriffon/src/mozilla-central-9db46ddfb517/toolkit/content/aboutTelemetry.js"
 const PREF_TELEMETRY_ENABLED = "toolkit.telemetry.enabled";
-//@line 29 "/build/bluegriffon/src/mozilla-central-9db46ddfb517/toolkit/content/aboutTelemetry.js"
 const PREF_DEBUG_SLOW_SQL = "toolkit.telemetry.debugSlowSql";
 const PREF_SYMBOL_SERVER_URI = "profiler.symbolicationUrl";
 const DEFAULT_SYMBOL_SERVER_URI = "http://symbolapi.mozilla.org";
 
+// ms idle before applying the filter (allow uninterrupted typing)
+const FILTER_IDLE_TIMEOUT = 500;
+
+//@line 38 "/build/bluegriffon/src/mozilla-central-fec90cbfbaad/toolkit/content/aboutTelemetry.js"
+const EOL = "\n";
+//@line 40 "/build/bluegriffon/src/mozilla-central-fec90cbfbaad/toolkit/content/aboutTelemetry.js"
+
 // Cached value of document's RTL mode
 let documentRTLMode = "";
 
@@ -100,6 +108,100 @@ let observer = {
   }
 };
 
+let GeneralData = {
+  /**
+   * Renders the general data
+   */
+  render: function() {
+    setHasData("general-data-section", true);
+
+    let table = document.createElement("table");
+
+    let caption = document.createElement("caption");
+    let captionString = bundle.GetStringFromName("generalDataTitle");
+    caption.appendChild(document.createTextNode(captionString + "\n"));
+    table.appendChild(caption);
+
+    let headings = document.createElement("tr");
+    this.appendColumn(headings, "th", bundle.GetStringFromName("generalDataHeadingName") + "\t");
+    this.appendColumn(headings, "th", bundle.GetStringFromName("generalDataHeadingValue") + "\t");
+    table.appendChild(headings);
+
+    let row = document.createElement("tr");
+    this.appendColumn(row, "td", "Client ID\t");
+    this.appendColumn(row, "td", TelemetryPing.clientID + "\t");
+    table.appendChild(row);
+
+    let dataDiv = document.getElementById("general-data");
+    dataDiv.appendChild(table);
+  },
+
+  /**
+   * Helper function for appending a column to the data table.
+   *
+   * @param aRowElement Parent row element
+   * @param aColType Column's tag name
+   * @param aColText Column contents
+   */
+  appendColumn: function(aRowElement, aColType, aColText) {
+    let colElement = document.createElement(aColType);
+    let colTextElement = document.createTextNode(aColText);
+    colElement.appendChild(colTextElement);
+    aRowElement.appendChild(colElement);
+  },
+};
+
+let TelLog = {
+  /**
+   * Renders the telemetry log
+   */
+  render: function() {
+    let entries =  TelemetryLog.entries();
+
+    if(entries.length == 0) {
+        return;
+    }
+    setHasData("telemetry-log-section", true);
+    let table = document.createElement("table");
+
+    let caption = document.createElement("caption");
+    let captionString = bundle.GetStringFromName("telemetryLogTitle");
+    caption.appendChild(document.createTextNode(captionString + "\n"));
+    table.appendChild(caption);
+
+    let headings = document.createElement("tr");
+    this.appendColumn(headings, "th", bundle.GetStringFromName("telemetryLogHeadingId") + "\t");
+    this.appendColumn(headings, "th", bundle.GetStringFromName("telemetryLogHeadingTimestamp") + "\t");
+    this.appendColumn(headings, "th", bundle.GetStringFromName("telemetryLogHeadingData") + "\t");
+    table.appendChild(headings);
+
+    for (let entry of entries) {
+        let row = document.createElement("tr");
+        for (let elem of entry) {
+            this.appendColumn(row, "td", elem + "\t");
+        }
+        table.appendChild(row);
+    }
+
+    let dataDiv = document.getElementById("telemetry-log");
+    dataDiv.appendChild(table);
+  },
+
+  /**
+   * Helper function for appending a column to the data table.
+   *
+   * @param aRowElement Parent row element
+   * @param aColType Column's tag name
+   * @param aColText Column contents
+   */
+  appendColumn: function(aRowElement, aColType, aColText) {
+    let colElement = document.createElement(aColType);
+    let colTextElement = document.createTextNode(aColText);
+    colElement.appendChild(colTextElement);
+    aRowElement.appendChild(colElement);
+  },
+};
+
 let SlowSQL = {
 
   slowSqlHits: bundle.GetStringFromName("slowSqlHits"),
@@ -123,10 +225,11 @@ let SlowSQL = {
     let mainThreadCount = Object.keys(mainThread).length;
     let otherThreadCount = Object.keys(otherThreads).length;
     if (mainThreadCount == 0 && otherThreadCount == 0) {
-      showEmptySectionMessage("slow-sql-section");
       return;
     }
 
+    setHasData("slow-sql-section", true);
+
     if (debugSlowSql) {
       document.getElementById("sql-warning").classList.remove("hidden");
     }
@@ -272,10 +375,11 @@ let StackRenderer = {
     }
 
     if (aStacks.length == 0) {
-      showEmptySectionMessage(aPrefix + '-section');
       return;
     }
 
+    setHasData(aPrefix + '-section', true);
+
     this.renderMemoryMap(div, aMemoryMap);
 
     for (let i = 0; i < aStacks.length; ++i) {
@@ -364,7 +468,7 @@ function SymbolicationRequest_fetchSymbols() {
                  "version" : 3};
   let requestJSON = JSON.stringify(request);
 
-  this.symbolRequest = XMLHttpRequest();
+  this.symbolRequest = new XMLHttpRequest();
   this.symbolRequest.open("POST", symbolServerURI, true);
   this.symbolRequest.setRequestHeader("Content-type", "application/json");
   this.symbolRequest.setRequestHeader("Content-length",
@@ -396,6 +500,56 @@ let ChromeHangs = {
   }
 };
 
+let ThreadHangStats = {
+
+  /**
+   * Renders raw thread hang stats data
+   */
+  render: function() {
+    let div = document.getElementById("thread-hang-stats");
+    clearDivData(div);
+
+    let stats = Telemetry.threadHangStats;
+    stats.forEach((thread) => {
+      div.appendChild(this.renderThread(thread));
+    });
+    if (stats.length) {
+      setHasData("thread-hang-stats-section", true);
+    }
+  },
+
+  /**
+   * Creates and fills data corresponding to a thread
+   */
+  renderThread: function(aThread) {
+    let div = document.createElement("div");
+
+    let title = document.createElement("h2");
+    title.textContent = aThread.name;
+    div.appendChild(title);
+
+    // Don't localize the histogram name, because the
+    // name is also used as the div element's ID
+    Histogram.render(div, aThread.name + "-Activity",
+                     aThread.activity, {exponential: true});
+    aThread.hangs.forEach((hang, index) => {
+      let hangName = aThread.name + "-Hang-" + (index + 1);
+      let hangDiv = Histogram.render(
+        div, hangName, hang.histogram, {exponential: true});
+      let stackDiv = document.createElement("div");
+      let stack = hang.nativeStack || hang.stack;
+      stack.forEach((frame) => {
+        stackDiv.appendChild(document.createTextNode(frame));
+        // Leave an extra <br> at the end of the stack listing
+        stackDiv.appendChild(document.createElement("br"));
+      });
+      // Insert stack after the histogram title
+      hangDiv.insertBefore(stackDiv, hangDiv.childNodes[1]);
+    });
+    return div;
+  },
+};
+
 let Histogram = {
 
   hgramSamplesCaption: bundle.GetStringFromName("histogramSamples"),
@@ -404,15 +558,20 @@ let Histogram = {
 
   hgramSumCaption: bundle.GetStringFromName("histogramSum"),
 
+  hgramCopyCaption: bundle.GetStringFromName("histogramCopy"),
+
   /**
    * Renders a single Telemetry histogram
    *
    * @param aParent Parent element
    * @param aName Histogram name
    * @param aHgram Histogram information
+   * @param aOptions Object with render options
+   *                 * exponential: bars follow logarithmic scale
    */
-  render: function Histogram_render(aParent, aName, aHgram) {
+  render: function Histogram_render(aParent, aName, aHgram, aOptions) {
     let hgram = this.unpack(aHgram);
+    let options = aOptions || {};
 
     let outerDiv = document.createElement("div");
     outerDiv.className = "histogram";
@@ -434,9 +593,22 @@ let Histogram = {
     if (isRTL())
       hgram.values.reverse();
 
-    this.renderValues(outerDiv, hgram.values, hgram.max);
+    let textData = this.renderValues(outerDiv, hgram.values, hgram.max,
+                                     hgram.sample_count, options);
+
+    // The 'Copy' button contains the textual data, copied to clipboard on click
+    let copyButton = document.createElement("button");
+    copyButton.className = "copy-node";
+    copyButton.appendChild(document.createTextNode(this.hgramCopyCaption));
+    copyButton.histogramText = aName + EOL + stats + EOL + EOL + textData;
+    copyButton.addEventListener("click", function(){
+      Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper)
+                                                 .copyString(this.histogramText);
+    });
+    outerDiv.appendChild(copyButton);
 
     aParent.appendChild(outerDiv);
+    return outerDiv;
   },
 
   /**
@@ -447,7 +619,7 @@ let Histogram = {
    * @return Unpacked histogram representation
    */
   unpack: function Histogram_unpack(aHgram) {
-    let sample_count = aHgram.counts.reduceRight(function (a, b) a + b);
+    let sample_count = aHgram.counts.reduceRight((a, b) => a + b);
     let buckets = [0, 1];
     if (aHgram.histogram_type != Telemetry.HISTOGRAM_BOOLEAN) {
       buckets = aHgram.ranges;
@@ -488,15 +660,44 @@ let Histogram = {
   },
 
   /**
-   * Create histogram bars
+   * Return a non-negative, logarithmic representation of a non-negative number.
+   * e.g. 0 => 0, 1 => 1, 10 => 2, 100 => 3
+   *
+   * @param aNumber Non-negative number
+   */
+  getLogValue: function(aNumber) {
+    return Math.max(0, Math.log10(aNumber) + 1);
+  },
+
+  /**
+   * Create histogram HTML bars, also returns a textual representation
+   * Both aMaxValue and aSumValues must be positive.
+   * Values are assumed to use 0 as baseline.
    *
    * @param aDiv Outer parent div
    * @param aValues Histogram values
-   * @param aMaxValue Largest histogram value in set
+   * @param aMaxValue Value of the longest bar (length, not label)
+   * @param aSumValues Sum of all bar values
+   * @param aOptions Object with render options (@see #render)
    */
-  renderValues: function Histogram_renderValues(aDiv, aValues, aMaxValue) {
+  renderValues: function Histogram_renderValues(aDiv, aValues, aMaxValue, aSumValues, aOptions) {
+    let text = "";
+    // If the last label is not the longest string, alignment will break a little
+    let labelPadTo = String(aValues[aValues.length -1][0]).length;
+    let maxBarValue = aOptions.exponential ? this.getLogValue(aMaxValue) : aMaxValue;
+
     for (let [label, value] of aValues) {
-      let belowEm = Math.round(MAX_BAR_HEIGHT * (value / aMaxValue) * 10) / 10;
+      let barValue = aOptions.exponential ? this.getLogValue(value) : value;
+
+      // Create a text representation: <right-aligned-label> |<bar-of-#><value>  <percentage>
+      text += EOL
+              + " ".repeat(Math.max(0, labelPadTo - String(label).length)) + label // Right-aligned label
+              + " |" + "#".repeat(Math.round(MAX_BAR_CHARS * barValue / maxBarValue)) // Bar
+              + "  " + value // Value
+              + "  " + Math.round(100 * value / aSumValues) + "%"; // Percentage
+
+      // Construct the HTML labels + bars
+      let belowEm = Math.round(MAX_BAR_HEIGHT * (barValue / maxBarValue) * 10) / 10;
       let aboveEm = MAX_BAR_HEIGHT - belowEm;
 
       let barDiv = document.createElement("div");
@@ -517,22 +718,119 @@ let Histogram = {
 
       aDiv.appendChild(barDiv);
     }
+
+    return text.substr(EOL.length); // Trim the EOL before the first line
+  },
+
+  /**
+   * Helper function for filtering histogram elements by their id
+   * Adds the "filter-blocked" class to histogram nodes whose IDs don't match the filter.
+   *
+   * @param aContainerNode Container node containing the histogram class nodes to filter
+   * @param aFilterText either text or /RegEx/. If text, case-insensitive and AND words
+   */
+  filterHistograms: function _filterHistograms(aContainerNode, aFilterText) {
+    let filter = aFilterText.toString();
+
+    // Pass if: all non-empty array items match (case-sensitive)
+    function isPassText(subject, filter) {
+      for (let item of filter) {
+        if (item.length && subject.indexOf(item) < 0) {
+          return false; // mismatch and not a spurious space
+        }
+      }
+      return true;
+    }
+
+    function isPassRegex(subject, filter) {
+      return filter.test(subject);
+    }
+
+    // Setup normalized filter string (trimmed, lower cased and split on spaces if not RegEx)
+    let isPassFunc; // filter function, set once, then applied to all elements
+    filter = filter.trim();
+    if (filter[0] != "/") { // Plain text: case insensitive, AND if multi-string
+      isPassFunc = isPassText;
+      filter = filter.toLowerCase().split(" ");
+    } else {
+      isPassFunc = isPassRegex;
+      var r = filter.match(/^\/(.*)\/(i?)$/);
+      try {
+        filter = RegExp(r[1], r[2]);
+      }
+      catch (e) { // Incomplete or bad RegExp - always no match
+        isPassFunc = function() {
+          return false;
+        };
+      }
+    }
+
+    let needLower = (isPassFunc === isPassText);
+
+    let histograms = aContainerNode.getElementsByClassName("histogram");
+    for (let hist of histograms) {
+      hist.classList[isPassFunc((needLower ? hist.id.toLowerCase() : hist.id), filter) ? "remove" : "add"]("filter-blocked");
+    }
+  },
+
+  /**
+   * Event handler for change at histograms filter input
+   *
+   * When invoked, 'this' is expected to be the filter HTML node.
+   */
+  histogramFilterChanged: function _histogramFilterChanged() {
+    if (this.idleTimeout) {
+      clearTimeout(this.idleTimeout);
+    }
+
+    this.idleTimeout = setTimeout( () => {
+      Histogram.filterHistograms(document.getElementById(this.getAttribute("target_id")), this.value);
+    }, FILTER_IDLE_TIMEOUT);
   }
 };
 
+/*
+ * Helper function to render JS objects with white space between top level elements
+ * so that they look better in the browser
+ * @param   aObject JavaScript object or array to render
+ * @return  String
+ */
+function RenderObject(aObject) {
+  let output = "";
+  if (Array.isArray(aObject)) {
+    if (aObject.length == 0) {
+      return "[]";
+    }
+    output = "[" + JSON.stringify(aObject[0]);
+    for (let i = 1; i < aObject.length; i++) {
+      output += ", " + JSON.stringify(aObject[i]);
+    }
+    return output + "]";
+  }
+  let keys = Object.keys(aObject);
+  if (keys.length == 0) {
+    return "{}";
+  }
+  output = "{\"" + keys[0] + "\":\u00A0" + JSON.stringify(aObject[keys[0]]);
+  for (let i = 1; i < keys.length; i++) {
+    output += ", \"" + keys[i] + "\":\u00A0" + JSON.stringify(aObject[keys[i]]);
+  }
+  return output + "}";
+};
+
 let KeyValueTable = {
-
-  keysHeader: bundle.GetStringFromName("keysHeader"),
-
-  valuesHeader: bundle.GetStringFromName("valuesHeader"),
-
   /**
-   * Fill out a 2-column table with keys and values
+   * Returns a 2-column table with keys and values
+   * @param aMeasurements Each key in this JS object is rendered as a row in
+   *                      the table with its corresponding value
+   * @param aKeysLabel    Column header for the keys column
+   * @param aValuesLabel  Column header for the values column
    */
-  render: function KeyValueTable_render(aTableID, aMeasurements) {
-    let table = document.getElementById(aTableID);
-    this.renderHeader(table);
+  render: function KeyValueTable_render(aMeasurements, aKeysLabel, aValuesLabel) {
+    let table = document.createElement("table");
+    this.renderHeader(table, aKeysLabel, aValuesLabel);
     this.renderBody(table, aMeasurements);
+    return table;
   },
 
   /**
@@ -540,15 +838,17 @@ let KeyValueTable = {
    * Tabs & newlines added to cells to make it easier to copy-paste.
    *
    * @param aTable Table element
+   * @param aKeysLabel    Column header for the keys column
+   * @param aValuesLabel  Column header for the values column
    */
-  renderHeader: function KeyValueTable_renderHeader(aTable) {
+  renderHeader: function KeyValueTable_renderHeader(aTable, aKeysLabel, aValuesLabel) {
     let headerRow = document.createElement("tr");
     aTable.appendChild(headerRow);
 
     let keysColumn = document.createElement("th");
-    keysColumn.appendChild(document.createTextNode(this.keysHeader + "\t"));
+    keysColumn.appendChild(document.createTextNode(aKeysLabel + "\t"));
     let valuesColumn = document.createElement("th");
-    valuesColumn.appendChild(document.createTextNode(this.valuesHeader + "\n"));
+    valuesColumn.appendChild(document.createTextNode(aValuesLabel + "\n"));
 
     headerRow.appendChild(keysColumn);
     headerRow.appendChild(valuesColumn);
@@ -563,8 +863,11 @@ let KeyValueTable = {
    */
   renderBody: function KeyValueTable_renderBody(aTable, aMeasurements) {
     for (let [key, value] of Iterator(aMeasurements)) {
-      if (typeof value == "object") {
-        value = JSON.stringify(value);
+      // use .valueOf() to unbox Number, String, etc. objects
+      if (value &&
+         (typeof value == "object") &&
+         (typeof value.valueOf() == "object")) {
+        value = RenderObject(value);
       }
 
       let newRow = document.createElement("tr");
@@ -581,35 +884,57 @@ let KeyValueTable = {
   }
 };
 
+let KeyedHistogram = {
+  render: function(parent, id, keyedHistogram) {
+    let outerDiv = document.createElement("div");
+    outerDiv.className = "keyed-histogram";
+    outerDiv.id = id;
+
+    let divTitle = document.createElement("div");
+    divTitle.className = "keyed-histogram-title";
+    divTitle.appendChild(document.createTextNode(id));
+    outerDiv.appendChild(divTitle);
+
+    for (let [name, hgram] of Iterator(keyedHistogram)) {
+      Histogram.render(outerDiv, name, hgram);
+    }
+
+    parent.appendChild(outerDiv);
+    return outerDiv;
+  },
+};
+
+let AddonDetails = {
+  tableIDTitle: bundle.GetStringFromName("addonTableID"),
+  tableDetailsTitle: bundle.GetStringFromName("addonTableDetails"),
+
+  /**
+   * Render the addon details section as a series of headers followed by key/value tables
+   * @param aSections Object containing the details sections to render
+   */
+  render: function AddonDetails_render(aSections) {
+    let addonSection = document.getElementById("addon-details");
+    for (let provider in aSections) {
+      let providerSection = document.createElement("h2");
+      let titleText = bundle.formatStringFromName("addonProvider", [provider], 1);
+      providerSection.appendChild(document.createTextNode(titleText));
+      addonSection.appendChild(providerSection);
+      addonSection.appendChild(
+        KeyValueTable.render(aSections[provider],
+                             this.tableIDTitle, this.tableDetailsTitle));
+    }
+  }
+};
+
 /**
- * Helper function for showing "No data collected" message for a section
+ * Helper function for showing either the toggle element or "No data collected" message for a section
  *
  * @param aSectionID ID of the section element that needs to be changed
+ * @param aHasData true (default) indicates that toggle should be displayed
  */
-function showEmptySectionMessage(aSectionID) {
+function setHasData(aSectionID, aHasData) {
   let sectionElement = document.getElementById(aSectionID);
-
-  // Hide toggle captions
-  let toggleElements = sectionElement.getElementsByClassName("toggle-caption");
-  toggleElements[0].classList.add("hidden");
-  toggleElements[1].classList.add("hidden");
-
-  // Show "No data collected" message
-  let messageElement = sectionElement.getElementsByClassName("empty-caption")[0];
-  messageElement.classList.remove("hidden");
-
-  // Don't allow section to be expanded by clicking on the header text
-  let sectionHeaders = sectionElement.getElementsByClassName("section-name");
-  for (let sectionHeader of sectionHeaders) {
-    sectionHeader.removeEventListener("click", toggleSection);
-    sectionHeader.style.cursor = "auto";
-  }
-
-  // Don't allow section to be expanded by clicking on the toggle text
-  let toggleLinks = sectionElement.getElementsByClassName("toggle-caption");
-  for (let toggleLink of toggleLinks) {
-    toggleLink.removeEventListener("click", toggleSection);
-  }
+  sectionElement.classList[aHasData ? "add" : "remove"]("has-data");
 }
 
 /**
@@ -618,12 +943,15 @@ function showEmptySectionMessage(aSectionID) {
  */
 function toggleSection(aEvent) {
   let parentElement = aEvent.target.parentElement;
-  let sectionDiv = parentElement.getElementsByTagName("div")[0];
-  sectionDiv.classList.toggle("hidden");
+  if (!parentElement.classList.contains("has-data")) {
+    return; // nothing to toggle
+  }
 
-  let toggleLinks = parentElement.getElementsByClassName("toggle-caption");
-  toggleLinks[0].classList.toggle("hidden");
-  toggleLinks[1].classList.toggle("hidden");
+  parentElement.classList.toggle("expanded");
+
+  // Store section opened/closed state in a hidden checkbox (which is then used on reload)
+  let statebox = parentElement.getElementsByClassName("statebox")[0];
+  statebox.checked = parentElement.classList.contains("expanded");
 }
 
 /**
@@ -676,7 +1004,7 @@ function setupListeners() {
 
   document.getElementById("late-writes-fetch-symbols").addEventListener("click",
     function () {
-      let lateWrites = TelemetryPing.getPayload().lateWrites;
+      let lateWrites = TelemetrySession.getPayload().lateWrites;
       let req = new SymbolicationRequest("late-writes",
                                          LateWritesSingleton.renderHeader,
                                          lateWrites.memoryMap,
@@ -686,18 +1014,17 @@ function setupListeners() {
 
   document.getElementById("late-writes-hide-symbols").addEventListener("click",
     function () {
-      let ping = TelemetryPing.getPayload();
+      let ping = TelemetrySession.getPayload();
       LateWritesSingleton.renderLateWrites(ping.lateWrites);
   }, false);
 
-
   // Clicking on the section name will toggle its state
   let sectionHeaders = document.getElementsByClassName("section-name");
   for (let sectionHeader of sectionHeaders) {
     sectionHeader.addEventListener("click", toggleSection, false);
   }
 
-  // Clicking on the "collapse"/"expand" text will also toggle section's state
+  // Clicking on the "toggle" text will also toggle section's state
   let toggleLinks = document.getElementsByClassName("toggle-caption");
   for (let toggleLink of toggleLinks) {
     toggleLink.addEventListener("click", toggleSection, false);
@@ -713,12 +1040,21 @@ function onLoad() {
   // Set up event listeners
   setupListeners();
 
+  // Show general data.
+  GeneralData.render();
+
+  // Show telemetry log.
+  TelLog.render();
+
   // Show slow SQL stats
   SlowSQL.render();
 
   // Show chrome hang stacks
   ChromeHangs.render();
 
+  // Show thread hang stats
+  ThreadHangStats.render();
+
   // Show histogram data
   let histograms = Telemetry.histogramSnapshots;
   if (Object.keys(histograms).length) {
@@ -726,8 +1062,25 @@ function onLoad() {
     for (let [name, hgram] of Iterator(histograms)) {
       Histogram.render(hgramDiv, name, hgram);
     }
-  } else {
-    showEmptySectionMessage("histograms-section");
+
+    let filterBox = document.getElementById("histograms-filter");
+    filterBox.addEventListener("input", Histogram.histogramFilterChanged, false);
+    if (filterBox.value.trim() != "") { // on load, no need to filter if empty
+      Histogram.filterHistograms(hgramDiv, filterBox.value);
+    }
+
+    setHasData("histograms-section", true);
+  }
+
+  // Show keyed histogram data
+  let keyedHistograms = Telemetry.keyedHistogramSnapshots;
+  if (Object.keys(keyedHistograms).length) {
+    let keyedDiv = document.getElementById("keyed-histograms");
+    for (let [id, keyed] of Iterator(keyedHistograms)) {
+      KeyedHistogram.render(keyedDiv, id, keyed);
+    }
+
+    setHasData("keyed-histograms-section", true);
   }
 
   // Show addon histogram data
@@ -741,13 +1094,21 @@ function onLoad() {
     }
   }
 
-  if (!addonHistogramsRendered) {
-    showEmptySectionMessage("addon-histograms-section");
+  if (addonHistogramsRendered) {
+   setHasData("addon-histograms-section", true);
   }
 
   // Get the Telemetry Ping payload
   Telemetry.asyncFetchTelemetryData(displayPingData);
-};
+
+  // Restore sections states
+  let stateboxes = document.getElementsByClassName("statebox");
+  for (let box of stateboxes) {
+    if (box.checked) { // Was open. Will still display as empty if not has-data
+        box.parentElement.classList.add("expanded");
+    }
+  }
+}
 
 let LateWritesSingleton = {
   renderHeader: function LateWritesSingleton_renderHeader(aIndex) {
@@ -770,21 +1131,20 @@ let LateWritesSingleton = {
  * @return Sorted measurements
  */
 function sortStartupMilestones(aSimpleMeasurements) {
-  // List of startup milestones
-  const startupMilestones =
-    ["start", "main", "startupCrashDetectionBegin", "createTopLevelWindow",
-     "firstPaint", "delayedStartupStarted", "firstLoadURI",
-     "sessionRestoreInitialized", "sessionRestoreRestoring", "sessionRestored",
-     "delayedStartupFinished", "startupCrashDetectionEnd",
-     "AMI_startup_begin", "AMI_startup_end", "XPI_startup_begin", "XPI_startup_end",
-     "XPI_bootstrap_addons_begin", "XPI_bootstrap_addons_end"];
+  const telemetryTimestamps = TelemetryTimestamps.get();
+  let startupEvents = Services.startup.getStartupInfo();
+  delete startupEvents['process'];
+
+  function keyIsMilestone(k) {
+    return (k in startupEvents) || (k in telemetryTimestamps);
+  }
 
   let sortedKeys = Object.keys(aSimpleMeasurements);
 
   // Sort the measurements, with startup milestones at the front + ordered by time
   sortedKeys.sort(function keyCompare(keyA, keyB) {
-    let isKeyAMilestone = (startupMilestones.indexOf(keyA) > -1);
-    let isKeyBMilestone = (startupMilestones.indexOf(keyB) > -1);
+    let isKeyAMilestone = keyIsMilestone(keyA);
+    let isKeyBMilestone = keyIsMilestone(keyB);
 
     // First order by startup vs non-startup measurement
     if (isKeyAMilestone && !isKeyBMilestone)
@@ -809,23 +1169,34 @@ function sortStartupMilestones(aSimpleMeasurements) {
 }
 
 function displayPingData() {
-  let ping = TelemetryPing.getPayload();
+  let ping = TelemetrySession.getPayload();
+
+  let keysHeader = bundle.GetStringFromName("keysHeader");
+  let valuesHeader = bundle.GetStringFromName("valuesHeader");
 
   // Show simple measurements
   let simpleMeasurements = sortStartupMilestones(ping.simpleMeasurements);
   if (Object.keys(simpleMeasurements).length) {
-    KeyValueTable.render("simple-measurements-table", simpleMeasurements);
-  } else {
-    showEmptySectionMessage("simple-measurements-section");
+    let simpleSection = document.getElementById("simple-measurements");
+    simpleSection.appendChild(KeyValueTable.render(simpleMeasurements,
+                                                   keysHeader, valuesHeader));
+    setHasData("simple-measurements-section", true);
   }
 
   LateWritesSingleton.renderLateWrites(ping.lateWrites);
 
   // Show basic system info gathered
   if (Object.keys(ping.info).length) {
-    KeyValueTable.render("system-info-table", ping.info);
-  } else {
-    showEmptySectionMessage("system-info-section");
+    let infoSection = document.getElementById("system-info");
+    infoSection.appendChild(KeyValueTable.render(ping.info,
+                                                 keysHeader, valuesHeader));
+    setHasData("system-info-section", true);
+  }
+
+  let addonDetails = ping.addonDetails;
+  if (Object.keys(addonDetails).length) {
+    AddonDetails.render(addonDetails);
+    setHasData("addon-details-section", true);
   }
 }
 
